import dartpy as dart

"""
Displays Contacts between a Robot arm in a GUI as a pointcloud

Demonstrates the following dartpy functionality:
- Extracting contact points from simulation
- Drawing of a point cloud (with variable data during simulation)
- Hooking functionality to be executed in the viewer run() function
"""


class ContactVisualizingNode(dart.gui.osg.RealTimeWorldNode):
    """ WorldNode subclass, which extracts the contacts of the last simulation
    step and adds them to a pointloud for visualization.

    Note: Simulation must run for the contacts to be displayed.

    """

    def __init__(self, world, pointCloudShape):
        """
        Input:
        ------
        world : the dart.simulation.World object to visualize
        pointCloudShape : dart.dynamics.PointCloudShape to set contacts in
                          before each display refresh
        """
        super(ContactVisualizingNode, self).__init__(world)
        self._pointCloudShape = pointCloudShape

    def customPreRefresh(self):
        # here, we overload the function to add contact points to the
        # pointcloud before each GUI refresh step
        contactPoints = self._getListOfContactPoints()
        self._pointCloudShape.setPoint(contactPoints)

    def _getListOfContactPoints(self):
        return [
            c.point
            for c in self.getWorld().getLastCollisionResult().getContacts()
        ]


def setAlpha(skeleton, alphaValue):
    """ setAlpha(skeleton : dartpy.dynamics.Skeleton, alphaValue : float) -> None

    sets the transparency (alpha) value for each body node of the skeleton,
    assuming that the VisualAspect is contained within the first ShapeNode
    by convention.

    Input:
    ------
    skeleton : The skeleton to set transparency values for
    alphaValue: [0,1] transparency, where 1 is fully visible
    """
    for bodyNode in skeleton.getBodyNodes():
        bodyNode.getShapeNode(0).getVisualAspect().setAlpha(alphaValue)


def main():
    world = dart.simulation.World()

    # load KR5 robot and ground plane, set transparent:
    urdfParser = dart.utils.DartLoader()
    kr5 = urdfParser.parseSkeleton("dart://sample/urdf/KR5/KR5 sixx R650.urdf")
    # set transparency, so that "inner" contact points will be visible
    setAlpha(kr5, 0.3)
    ground = urdfParser.parseSkeleton("dart://sample/urdf/KR5/ground.urdf")
    setAlpha(ground, 0.3)
    world.addSkeleton(kr5)
    world.addSkeleton(ground)
    world.setGravity([0, -9.81, 0])

    # add point cloud shape for visualizing contacts
    pointCloudShape = dart.dynamics.PointCloudShape(0.02)

    # Since contact points may change during execution, dynamic data variance
    # is assumed for the pointcloud of contacts. Otherwise, OSG will not render
    # the new points.
    pointCloudShape.setDataVariance(dart.dynamics.Shape.DataVariance.DYNAMIC)

    pointCloudSimpleFrame = dart.dynamics.SimpleFrame(
        dart.dynamics.Frame.World(), "ContactsVisualization"
    )
    pointCloudSimpleFrame.setShape(pointCloudShape)
    pcVisualAspect = pointCloudSimpleFrame.createVisualAspect()
    pcVisualAspect.setRGBA([0.7, 0, 0, 1])
    world.addSimpleFrame(pointCloudSimpleFrame)

    # Create world node and add it to viewer
    node = ContactVisualizingNode(world, pointCloudShape)

    # create a viewer with background color (red, green, blue, alpha), here: white
    viewer = dart.gui.osg.Viewer([1.0, 1.0, 1.0, 1.0])
    viewer.addWorldNode(node)

    # Grid settings
    grid = dart.gui.osg.GridVisual()
    grid.setPlaneType(dart.gui.osg.GridVisual.PlaneType.ZX)
    grid.setOffset([0, -0.55, 0])
    viewer.addAttachment(grid)

    viewer.setUpViewInWindow(0, 0, 640, 480)
    viewer.setCameraHomePosition(
        [2.0, 1.0, 2.0], [0.00, 0.00, 0.00], [-0.24, 0.94, -0.25]
    )
    viewer.run()


if __name__ == "__main__":
    main()



import pytest
import math
import numpy as np
import dartpy as dart


def read_world(uri: dart.common.Uri):
    options = dart.utils.SdfParser.Options()
    options.mDefaultRootJointType = dart.utils.SdfParser.RootJointType.FIXED
    world = dart.utils.SdfParser.readWorld(uri, options)
    for i in range(world.getNumSkeletons()):
        skel = world.getSkeleton(i)
        for j in range(skel.getNumJoints()):
            joint = skel.getJoint(j)
            joint.setLimitEnforcement(True)
    return world


def test_static():
    # Load world
    world = read_world("dart://sample/sdf/test/force_torque_test.world")
    assert world is not None
    assert world.getNumSkeletons() == 1

    # Check if the world is correctly loaded
    model_1 = world.getSkeleton(0)
    assert model_1 is not None
    assert model_1.getName() == "model_1"
    assert model_1.getNumBodyNodes() == 2
    assert model_1.getNumJoints() == 2

    world.setGravity([0, 0, -50])

    # Simulate 1 step
    world.step()
    t = world.getTime()

    # Get time step size
    dt = world.getTimeStep()
    assert dt > 0
    assert t == pytest.approx(dt)

    # Get joint and get force torque
    link_1 = model_1.getBodyNode("link_1")
    assert link_1 is not None
    link_2 = model_1.getBodyNode("link_2")
    assert link_2 is not None
    joint_01 = model_1.getJoint("joint_01")
    assert joint_01 is not None
    joint_12 = model_1.getJoint("joint_12")
    assert joint_12 is not None

    tf = dart.math.Isometry3()
    
    # Run 10 steps
    for _ in range(10):
        world.step()

        #----------------------
        # Test joint_01 wrench
        #----------------------

        # Reference adjustment for the difference of the joint frame conventions
        # between Gazebo and DART
        tf.set_identity()
        tf.set_translation(joint_01.getTransformFromParentBodyNode().translation())
        parent_frame01 = dart.dynamics.SimpleFrame(dart.dynamics.Frame.World(), "parent_frame01", tf)
        tf.set_identity()
        tf.set_translation(joint_01.getTransformFromChildBodyNode().translation())
        child_frame01 = dart.dynamics.SimpleFrame(link_1, "child_frame01", tf)

        parent_f01 = joint_01.getWrenchToParentBodyNode(parent_frame01)
        assert (parent_f01 == [0, 0, 0, 0, 0, 1000]).all()

        child_f01 = joint_01.getWrenchToChildBodyNode(child_frame01)
        assert (child_f01 == -parent_f01).all()

        #----------------------
        # Test joint_12 wrench
        #----------------------

        # Reference adjustment for the difference of the joint frame conventions
        # between Gazebo and DART
        tf.set_identity()
        tf.set_translation(joint_12.getTransformFromParentBodyNode().translation())
        parent_frame12 = dart.dynamics.SimpleFrame(link_1, "parent_frame12", tf)
        tf.set_identity()
        tf.set_translation(joint_12.getTransformFromChildBodyNode().translation())
        child_frame12 = dart.dynamics.SimpleFrame(link_2, "child_frame12", tf)

        parent_f12 = joint_12.getWrenchToParentBodyNode(parent_frame12)
        assert (parent_f12 == [0, 0, 0, 0, 0, 500]).all()

        child_f12 = joint_12.getWrenchToChildBodyNode(child_frame12)
        assert (child_f12 == -parent_f12).all()


def test_force_torque_at_joint_limits():
    # Load world
    world = read_world("dart://sample/sdf/test/force_torque_test.world")
    assert world is not None
    assert world.getNumSkeletons() == 1

    # Check if the world is correctly loaded
    model_1 = world.getSkeleton(0)
    assert model_1 is not None
    assert model_1.getName() == "model_1"
    assert model_1.getNumBodyNodes() == 2
    assert model_1.getNumJoints() == 2

    world.setGravity([0, 0, -50])

    # Simulate 1 step
    world.step()
    t = world.getTime()

    # Get time step size
    dt = world.getTimeStep()
    assert dt > 0
    assert t == pytest.approx(dt)

    # Get joint and get force torque
    link_1 = model_1.getBodyNode("link_1")
    assert link_1 is not None
    link_2 = model_1.getBodyNode("link_2")
    assert link_2 is not None
    joint_01 = model_1.getJoint("joint_01")
    assert joint_01 is not None
    joint_12 = model_1.getJoint("joint_12")
    assert joint_12 is not None

    # Change gravity so that the top link topples over, then remeasure
    world.setGravity([-30, 10, -50])

    # Wait for dynamics to be stabilized
    for i in range(2000):
        world.step()

    tf = dart.math.Isometry3()
    
    # Run 5 steps
    for _ in range(5):
        world.step()

        #----------------------
        # Test joint_01 wrench
        #----------------------

        # Reference adjustment for the difference of the joint frame conventions
        # between Gazebo and DART
        tf.set_identity()
        tf.set_translation(joint_01.getTransformFromParentBodyNode().translation())
        parent_frame01 = dart.dynamics.SimpleFrame(dart.dynamics.Frame.World(), "parent_frame01", tf)
        tf.set_identity()
        tf.set_translation(joint_01.getTransformFromChildBodyNode().translation())
        child_frame01 = dart.dynamics.SimpleFrame(link_1, "child_frame01", tf)

        parent_f01 = joint_01.getWrenchToParentBodyNode(parent_frame01)
        assert np.isclose(parent_f01, [750, 0, -450, 600, -200, 1000], rtol=0.1, atol=4.5).all()

        child_f01 = joint_01.getWrenchToChildBodyNode(child_frame01)
        assert np.isclose(child_f01, [-750, -450, 0, -600, 1000, 200], rtol=0.1, atol=4.5).all()

        #----------------------
        # Test joint_12 wrench
        #----------------------

        # Reference adjustment for the difference of the joint frame conventions
        # between Gazebo and DART
        tf.set_identity()
        tf.set_translation(joint_12.getTransformFromParentBodyNode().translation())
        parent_frame12 = dart.dynamics.SimpleFrame(link_1, "parent_frame12", tf)
        tf.set_identity()
        tf.set_translation(joint_12.getTransformFromChildBodyNode().translation())
        child_frame12 = dart.dynamics.SimpleFrame(link_2, "child_frame12", tf)

        parent_f12 = joint_12.getWrenchToParentBodyNode(parent_frame12)
        assert np.isclose(parent_f12, [250, 150, 0, 300, -500, -100], rtol=0.1, atol=0.1).all()

        child_f12 = joint_12.getWrenchToChildBodyNode(child_frame12)
        assert np.isclose(child_f12, [-250, -150, 0, -300, 500, 100], rtol=0.1, atol=0.1).all()


def test_force_torque_at_joint_limits_with_external_forces():
    # Load world
    world = read_world("dart://sample/sdf/test/force_torque_test2.world")
    assert world is not None
    assert world.getNumSkeletons() == 1

    # Check if the world is correctly loaded
    model_1 = world.getSkeleton(0)
    assert model_1 is not None
    assert model_1.getName() == "boxes"
    assert model_1.getNumBodyNodes() == 3
    assert model_1.getNumJoints() == 3
    assert model_1.getNumDofs() == 2

    # The first joint is fixed joint
    assert model_1.getRootJoint().getType() == dart.dynamics.WeldJoint.getStaticType()

    world.setGravity([0, 0, -50])

    # Simulate 1 step
    world.step()
    t = world.getTime()

    # Get time step size
    dt = world.getTimeStep()
    assert dt > 0
    assert t == pytest.approx(dt)

    # Get joint and get force torque
    link_1 = model_1.getBodyNode("link1")
    assert link_1 is not None
    link_2 = model_1.getBodyNode("link2")
    assert link_2 is not None
    link_3 = model_1.getBodyNode("link3")
    assert link_3 is not None
    joint_12 = model_1.getJoint("joint1")
    assert joint_12 is not None
    joint_23 = model_1.getJoint("joint2")
    assert joint_23 is not None

    tf = dart.math.Isometry3()
    
    # Run 45005 steps
    kp1 = 5e+4
    kp2 = 1e+4
    target1 = 0
    target2 = -0.25 * math.pi
    steps = 4500
    for _ in range(steps):
        # PD control
        j1_state = joint_12.getPosition(0)
        j2_state = joint_23.getPosition(0)
        p1_error = target1 - j1_state
        p2_error = target2 - j2_state
        effort1 = kp1 * p1_error
        effort2 = kp2 * p2_error
        joint_12.setForce(0, effort1)
        joint_23.setForce(0, effort2)

        world.step()

    assert joint_12.getPosition(0) == pytest.approx(target1, abs=1e-1)
    assert joint_23.getPosition(0) == pytest.approx(target2, abs=1e-1)

    tol = 2

    #----------------------
    # Test joint_12 wrench
    #----------------------

    # Reference adjustment for the difference of the joint frame conventions
    # between Gazebo and DART
    tf.set_identity()
    tf.set_translation(joint_12.getTransformFromParentBodyNode().translation())
    parent_frame01 = dart.dynamics.SimpleFrame(link_1, "parent_frame01", tf)
    tf.set_identity()
    tf.set_translation(joint_12.getTransformFromChildBodyNode().translation())
    child_frame01 = dart.dynamics.SimpleFrame(link_2, "child_frame01", tf)

    parent_f01 = joint_12.getWrenchToParentBodyNode(parent_frame01)
    assert np.isclose(parent_f01, [25, -175, 0, 0, 0, 300], rtol=0.01, atol=tol).all()

    child_f01 = joint_12.getWrenchToChildBodyNode(child_frame01)
    assert np.isclose(child_f01, [-25, 175, 0, 0, 0, -300], rtol=0.01, atol=tol).all()

    #----------------------
    # Test joint_23 wrench
    #----------------------

    # Reference adjustment for the difference of the joint frame conventions
    # between Gazebo and DART
    tf.set_identity()
    tf.set_translation(joint_23.getTransformFromParentBodyNode().translation())
    parent_frame12 = dart.dynamics.SimpleFrame(link_2, "parent_frame12", tf)
    tf.set_identity()
    tf.set_translation(joint_23.getTransformFromChildBodyNode().translation())
    child_frame12 = dart.dynamics.SimpleFrame(link_3, "child_frame12", tf)

    parent_f12 = joint_23.getWrenchToParentBodyNode(parent_frame12)
    assert np.isclose(parent_f12, [25, 0, 0, 0, 0, 50], rtol=0.01, atol=tol).all()

    child_f12 = joint_23.getWrenchToChildBodyNode(child_frame12)
    assert np.isclose(child_f12, [-17.678, 0, 17.679, -35.355, 0, -35.355], rtol=0.01, atol=tol).all()




if __name__ == "__main__":
    pytest.main()



import math
import numpy as np
import dartpy as dart


default_height = 1.0 # m
default_width = 0.2  # m
default_depth = 0.2  # m

default_torque = 15.0 # N-m
default_force = 15.0  # N
default_countdown = 200  # Number of timesteps for applying force

default_rest_position = 0.0
delta_rest_position = 10.0 * math.pi / 180.0

default_stiffness = 0.0
delta_stiffness = 10

default_damping = 5.0
delta_damping = 1.0


def set_geometry(body):
    # Create a BoxShape to be used for both visualization and collision checking
    box = dart.dynamics.BoxShape([default_width, default_depth, default_height])

    # Create a shape node for visualization and collision checking
    shape_node = body.createShapeNode(box)
    visual = shape_node.createVisualAspect()
    shape_node.createCollisionAspect()
    shape_node.createDynamicsAspect()
    visual.setColor([0, 0, 1])

    # Set the location of the shape node
    box_tf = dart.math.Isometry3()
    center = [0, 0, default_height / 2.0]
    box_tf.set_translation(center)
    shape_node.setRelativeTransform(box_tf)

    # Move the center of mass to the center of the object
    body.setLocalCOM(center)


def make_root_body(chain, name):
    joint_prop = dart.dynamics.BallJointProperties()
    joint_prop.mName = name + '_joint'
    joint_prop.mRestPositions = np.ones(3) * default_rest_position
    joint_prop.mSpringStiffnesses = np.ones(3) * default_stiffness
    joint_prop.mDampingCoefficients = np.ones(3) * default_damping

    body_aspect_prop = dart.dynamics.BodyNodeAspectProperties(name)
    body_prop = dart.dynamics.BodyNodeProperties(body_aspect_prop)

    [joint, body] = chain.createBallJointAndBodyNodePair(None, joint_prop, body_prop)

    # Make a shape for the Joint
    r = default_width
    ball = dart.dynamics.EllipsoidShape(math.sqrt(2) * np.ones(3) * r)
    shape_node = body.createShapeNode(ball)
    visual = shape_node.createVisualAspect()
    visual.setColor([0, 0, 1, 0.8])

    # Set the geometry of the BodyNode
    set_geometry(body)

    return body


def add_body(chain, parent, name):
    # Set up the properties for the Joint
    joint_prop = dart.dynamics.RevoluteJointProperties()
    joint_prop.mName = name + "_joint"
    joint_prop.mAxis = [0, 1, 0]
    joint_prop.mT_ParentBodyToJoint.set_translation([0, 0, default_height])

    # Set up the properties for the BodyNode
    body_aspect_prop = dart.dynamics.BodyNodeAspectProperties(name)
    body_prop = dart.dynamics.BodyNodeProperties(body_aspect_prop)

    # Create a new BodyNode, attached to its parent by a RevoluteJoint
    [joint, body] = chain.createRevoluteJointAndBodyNodePair(parent, joint_prop, body_prop)

    joint.setRestPosition(0, default_rest_position)
    joint.setSpringStiffness(0, default_stiffness)
    joint.setDampingCoefficient(0, default_damping)

    # Make a shape for the Joint
    r = default_width / 2.0
    h = default_depth
    cylinder = dart.dynamics.CylinderShape(r, h)

    # Line up the cylinder with the Joint axis
    tf = dart.math.Isometry3()
    angles = [math.pi / 2, 0, 0]

    rot = dart.math.eulerXYZToMatrix(angles)
    tf.set_rotation(rot)

    shape_node = body.createShapeNode(cylinder)
    visual = shape_node.createVisualAspect()
    visual.setColor([0, 0, 1, 0.8])
    shape_node.setRelativeTransform(tf)

    # Set the geometry of the Body
    set_geometry(body)

    return body


class MyWorldNode(dart.gui.osg.RealTimeWorldNode):
    def __init__(self, world):
        super(MyWorldNode, self).__init__(world)
        pass

    def customPreStep(self):
        pass


def main():
    # Create an empty Skeleton with the name 'chain'
    chain = dart.dynamics.Skeleton('chain')

    # Add each body to the last BodyNode in the pendulum
    body = make_root_body(chain, 'body1')
    body = add_body(chain, body, 'body2')
    body = add_body(chain, body, 'body3')
    body = add_body(chain, body, 'body4')
    body = add_body(chain, body, 'body5')

    # Set the initial position of the first DegreeOfFreedom so that the pendulum
    # starts to swing right away
    chain.setPosition(1, 120 * math.pi / 180.0);

    world = dart.simulation.World()
    world.addSkeleton(chain)

    node = MyWorldNode(world)

    # Create world node and add it to viewer
    viewer = dart.gui.osg.Viewer()
    viewer.addWorldNode(node)

    # Grid settings
    grid = dart.gui.osg.GridVisual()
    grid.setPlaneType(dart.gui.osg.GridVisual.PlaneType.XY)
    grid.setOffset([0, 0, -5])
    viewer.addAttachment(grid)

    viewer.setUpViewInWindow(0, 0, 1280, 760)
    viewer.setCameraHomePosition([8.0, 8.0, 4.0], [0, 0, -2.5], [0, 0, 1])
    viewer.run()


if __name__ == "__main__":
    main()



import platform

import pytest
import dartpy
from dartpy.utils import DartLoader
import os

from tests.util import get_asset_path


def test_parse_skeleton_non_existing_path_returns_null():
    assert os.path.isfile(get_asset_path('skel/cubes.skel')) is True
    loader = DartLoader()
    assert loader.parseSkeleton(get_asset_path('skel/test/does_not_exist.urdf')) is None


def test_parse_skeleton_invalid_urdf_returns_null():
    loader = DartLoader()
    assert loader.parseSkeleton(get_asset_path('urdf/invalid.urdf')) is None


def test_parse_skeleton_missing_mesh_returns_null():
    loader = DartLoader()
    assert loader.parseSkeleton(get_asset_path('urdf/missing_mesh.urdf')) is None


def test_parse_skeleton_invalid_mesh_returns_null():
    loader = DartLoader()
    assert loader.parseSkeleton(get_asset_path('urdf/invalid_mesh.urdf')) is None


def test_parse_skeleton_missing_package_returns_null():
    loader = DartLoader()
    assert loader.parseSkeleton(get_asset_path('urdf/missing_package.urdf')) is None


def test_parse_skeleton_loads_primitive_geometry():
    loader = DartLoader()
    assert loader.parseSkeleton(get_asset_path('urdf/test/primitive_geometry.urdf')) is not None


# Failing with following errors:
# TypeError: No to_python (by-value) converter found for C++ type: std::shared_ptr<dart::simulation::World>
#
# def test_parse_world():
#     loader = DartLoader()
#     assert loader.parseWorld(get_asset_path('urdf/testWorld.urdf')) is not None


def test_parse_joint_properties():
    loader = DartLoader()
    robot = loader.parseSkeleton(get_asset_path('urdf/test/joint_properties.urdf'))
    assert robot is not None

#    joint1 = robot.getJoint(1)
#    assert joint1 is not None
#    assert joint1.getDampingCoefficient(0) == pytest.approx(1.2, 1e-12)
#    assert joint1.getCoulombFriction(0) == pytest.approx(2.3, 1e-12)

#    joint2 = robot.getJoint(2)
#    assert joint2 is not None
#    assert joint2.getPositionLowerLimit(0) == -float("inf")
#    assert joint2.getPositionUpperLimit(0) == float("inf")
#    if not platform.linux_distribution()[1] == '14.04':
#        assert joint2.isCyclic(0)


if __name__ == "__main__":
    pytest.main()



# Copyright (c) 2011-2021, The DART development contributors
# All rights reserved.
#
# The list of contributors can be found at:
#   https://github.com/dartsim/dart/blob/master/LICENSE
#
# This file is provided under the "BSD-style" License

import pytest
import dartpy as dart


def test_read_world():
    assert dart.utils.SkelParser.readWorld(
        'dart://sample/skel/cubes.skel') is not None
    assert dart.utils.SkelParser.readWorld(
        'dart://sample/skel/cubes.skel', None) is not None


if __name__ == "__main__":
    pytest.main()



import platform
import pytest
import math
import numpy as np
import dartpy as dart


def test_solve_for_free_joint():
    '''
    Very simple test of InverseKinematics module, applied to a FreeJoint to
    ensure that the target is reachable
    '''

    skel = dart.dynamics.Skeleton()
    [joint0, body0] = skel.createFreeJointAndBodyNodePair()

    ik = body0.getOrCreateIK()
    assert ik.isActive()

    tf = dart.math.Isometry3()
    tf.set_translation([0, 0, 0.8])
    tf.set_rotation(dart.math.AngleAxis(math.pi / 8.0, [0, 1, 0]).to_rotation_matrix())
    ik.getTarget().setTransform(tf)

    error_method = ik.getErrorMethod()
    assert error_method.getMethodName() == 'TaskSpaceRegion'
    [lb, ub] = error_method.getBounds()
    assert len(lb) is 6
    assert len(ub) is 6
    error_method.setBounds(np.ones(6) * -1e-8, np.ones(6) * 1e-8)
    [lb, ub] = error_method.getBounds()
    assert lb == pytest.approx(-1e-8)
    assert ub == pytest.approx(1e-8)

    solver = ik.getSolver()
    solver.setNumMaxIterations(100)

    prob = ik.getProblem()

    tf_actual = ik.getTarget().getTransform().matrix()
    tf_expected = body0.getTransform().matrix()
    assert not np.isclose(tf_actual, tf_expected).all()

    success = solver.solve()
    assert success

    tf_actual = ik.getTarget().getTransform().matrix()
    tf_expected = body0.getTransform().matrix()
    assert np.isclose(tf_actual, tf_expected).all()


class FailingSolver(dart.optimizer.Solver):
    def __init__(self, constant):
        super(FailingSolver, self).__init__()
        self.constant = constant

    def solve(self):
        problem = self.getProblem()
        if problem is None:
            print('[FailingSolver::solve] Attempting to solve a nullptr problem! We will return false.')
            return False

        dim = problem.getDimension()
        wrong_solution = np.ones(dim) * self.constant
        problem.setOptimalSolution(wrong_solution)

        return False

    def getType(self):
        return 'FailingSolver'

    def clone(self):
        return FailingSolver(self.constant)


def test_do_not_apply_solution_on_failure():
    skel = dart.dynamics.Skeleton()
    [joint, body] = skel.createFreeJointAndBodyNodePair()

    ik = body.getIK(True)
    solver = FailingSolver(10)
    ik.setSolver(solver)

    dofs = skel.getNumDofs()
    skel.resetPositions()

    assert not ik.solveAndApply(allowIncompleteResult=False)
    assert np.isclose(skel.getPositions(), np.zeros(dofs)).all()

    assert not ik.solveAndApply(allowIncompleteResult=True)
    assert not np.isclose(skel.getPositions(), np.zeros(dofs)).all()


if __name__ == "__main__":
    pytest.main()



import numpy as np
import dartpy as dart


class InputHandler(dart.gui.osg.GUIEventHandler):
    def __init__(self, node):
        super(InputHandler, self).__init__()
        self.node = node
        self.force = np.zeros(3)
        self.impulse_duration = 0

    def handle(self, ea, aa):
        if ea.getEventType() == dart.gui.osg.GUIEventAdapter.KEYDOWN:
            if ea.getKey() == dart.gui.osg.GUIEventAdapter.KEY_1:
                ext_force = np.zeros(3)
                ext_force[0] = 40
                self.node.set_external_force(ext_force, 100)
                return True
            if ea.getKey() == dart.gui.osg.GUIEventAdapter.KEY_2:
                ext_force = np.zeros(3)
                ext_force[0] = -40
                self.node.set_external_force(ext_force, 100)
                return True
            if ea.getKey() == dart.gui.osg.GUIEventAdapter.KEY_3:
                ext_force = np.zeros(3)
                ext_force[2] = 40
                self.node.set_external_force(ext_force, 100)
                return True
            if ea.getKey() == dart.gui.osg.GUIEventAdapter.KEY_4:
                ext_force = np.zeros(3)
                ext_force[2] = -40
                self.node.set_external_force(ext_force, 100)
                return True

        return False


class MyWorldNode(dart.gui.osg.RealTimeWorldNode):
    def __init__(self, world, skel):
        super(MyWorldNode, self).__init__(world)
        self.world = world
        self.skel = skel
        self.dofs = self.skel.getNumDofs()
        self.left_heel = self.skel.getBodyNode('h_heel_left')
        self.left_foot = [
            self.skel.getDof('j_heel_left_1').getIndexInSkeleton(),
            self.skel.getDof('j_toe_left').getIndexInSkeleton(),
        ]
        self.right_foot = [
            self.skel.getDof('j_heel_right_1').getIndexInSkeleton(),
            self.skel.getDof('j_toe_right').getIndexInSkeleton(),
        ]
        self.timestep = world.getTimeStep()
        self.Kp = np.eye(self.dofs)
        self.Kd = np.eye(self.dofs)

        self.torques = np.zeros(self.dofs)

        self.q_d = self.skel.getPositions()

        # Using SPD results in simple Kp coefficients
        for i in range(6):
            self.Kp[i, i] = 0.0
            self.Kd[i, i] = 0.0

        for i in range(6, self.dofs):
            self.Kp[i, i] = 400.0
            self.Kd[i, i] = 40.0

        self.pre_offset = 0

        self.ext_force = np.zeros(3)
        self.ext_force_duration = 0

        self.ext_force_arrow_shape = dart.dynamics.ArrowShape([0, 0, 0], [0, 0, 0])

        self.ext_force_simple_frame = dart.dynamics.SimpleFrame()
        self.ext_force_simple_frame.setShape(self.ext_force_arrow_shape)
        self.ext_force_visual = self.ext_force_simple_frame.createVisualAspect()
        self.ext_force_visual.setColor([1.0, 0.0, 0.0])
        self.ext_force_visual.hide()

        self.world.addSimpleFrame(self.ext_force_simple_frame)

    def customPreStep(self):
        q = self.skel.getPositions();
        dq = self.skel.getVelocities();
        constraint_forces = self.skel.getConstraintForces()

        # SPD tracking
        invM = np.linalg.inv(self.skel.getMassMatrix() + self.Kd * self.timestep)
        p = np.matmul(-self.Kp, q + dq * self.timestep - self.q_d)
        d = np.matmul(-self.Kd, dq)
        ddq = np.matmul(invM, -self.skel.getCoriolisAndGravityForces() + p + d + constraint_forces)

        self.torques = p + d + np.matmul(-self.Kd, ddq) * self.timestep

        # Ankle strategy for sagital plane
        com = self.skel.getCOM()
        cop = self.left_heel.getTransform().multiply([0.05, 0, 0])

        offset = com[0] - cop[0]
        if offset < 0.1 and offset > 0.0:
             k1 = 200
             k2 = 100
             kd = 10
             self.torques[self.left_foot[0]] += -k1 * offset + kd * (self.pre_offset - offset)
             self.torques[self.left_foot[1]] += -k2 * offset + kd * (self.pre_offset - offset)
             self.torques[self.right_foot[0]] += -k1 * offset + kd * (self.pre_offset - offset)
             self.torques[self.right_foot[1]] += -k2 * offset + kd * (self.pre_offset - offset)
        elif offset > -0.2 and offset < -0.05:
            k1 = 2000
            k2 = 100
            kd = 100
            self.torques[self.left_foot[0]] += -k1 * offset + kd * (self.pre_offset - offset)
            self.torques[self.left_foot[1]] += -k2 * offset + kd * (self.pre_offset - offset)
            self.torques[self.right_foot[0]] += -k1 * offset + kd * (self.pre_offset - offset)
            self.torques[self.right_foot[1]] += -k2 * offset + kd * (self.pre_offset - offset)

        # Just to make sure no illegal torque is used
        for i in range(6):
            self.torques[i] = 0

        self.skel.setForces(self.torques * 0.8)

        # Apply external force
        self.ext_force_duration = self.ext_force_duration - 1
        if self.ext_force_duration <= 0:
            self.ext_force_duration = 0
            self.ext_force = np.zeros(3)
        spine = self.skel.getBodyNode('h_spine')
        spine.addExtForce(self.ext_force)
        if self.ext_force_duration > 0:
            arrow_head = spine.getTransform().translation()
            arrow_tail = arrow_head - self.ext_force / 30
            self.ext_force_arrow_shape.setPositions(arrow_tail, arrow_head)
            self.ext_force_arrow_shape.setDataVariance(dart.dynamics.Shape.DYNAMIC)
            self.ext_force_visual.show()
        else:
            self.ext_force_arrow_shape.setDataVariance(dart.dynamics.Shape.STATIC)
            self.ext_force_visual.hide()

    def set_external_force(self, force, duration=10):
        self.ext_force = force
        self.ext_force_duration = duration

    def external_force(self):
        return self.ext_force


def main():
    world = dart.utils.SkelParser.readWorld('dart://sample/skel/fullbody1.skel')
    world.setGravity([0, -9.81, 0])

    biped = world.getSkeleton('fullbody1')
    biped.getDof('j_pelvis_rot_y').setPosition(-0.20)
    biped.getDof('j_thigh_left_z').setPosition(0.15)
    biped.getDof('j_shin_left').setPosition(-0.40)
    biped.getDof('j_heel_left_1').setPosition(0.25)
    biped.getDof('j_thigh_right_z').setPosition(0.15)
    biped.getDof('j_shin_right').setPosition(-0.40)
    biped.getDof('j_heel_right_1').setPosition(0.25)
    biped.getDof('j_abdomen_2').setPosition(0.00)

    node = MyWorldNode(world, biped)

    # Create world node and add it to viewer
    viewer = dart.gui.osg.Viewer()
    viewer.addWorldNode(node)

    input_handler = InputHandler(node)
    viewer.addEventHandler(input_handler)

    viewer.setUpViewInWindow(0, 0, 640, 480)
    viewer.setCameraHomePosition([3, 1.5, 3], [0, 0, 0], [0, 0, 0])
    viewer.run()


if __name__ == "__main__":
    main()



import platform
import pytest
from dartpy.math import Random


def test_create():
    rand = Random()


def test_seed():
    rand = Random()

    N = 10

    min = -10
    max = 10

    first = []
    second = []
    third = []

    tol = 1e-6

    for i in range(N):
        Random.setSeed(i)
        first.append(Random.uniform(min, max));
        second.append(Random.uniform(min, max));
        third.append(Random.uniform(min, max));

    for i in range(N):
        Random.setSeed(i)
        assert Random.getSeed() is i
        assert Random.uniform(min, max) == pytest.approx(first[i], tol)
        assert Random.uniform(min, max) == pytest.approx(second[i], tol)
        assert Random.uniform(min, max) == pytest.approx(third[i], tol)


if __name__ == "__main__":
    pytest.main()



# Copyright (c) 2011-2021, The DART development contributors
# All rights reserved.
#
# The list of contributors can be found at:
#   https://github.com/dartsim/dart/blob/master/LICENSE
#
# This file is provided under the "BSD-style" License

import pytest
import dartpy as dart


def test_read_world():
    assert dart.utils.SdfParser.readWorld(
        'dart://sample/sdf/double_pendulum.world') is not None


if __name__ == "__main__":
    pytest.main()



import platform
import pytest
from dartpy.common import Uri


def test_from_string_valid_uri_returns_true():
    uri = Uri()
    assert uri.fromString("ftp://ftp.is.co.za/rfc/rfc1808.txt") is True
    assert uri.fromString("http://www.ietf.org/rfc/rfc2396.txt") is True
    assert uri.fromString("ldap://[2001:db8::7]/c=GB?objectClass?one") is True
    assert uri.fromString("mailto:John.Doe@example.com") is True
    assert uri.fromString("news:comp.infosystems.www.servers.unix") is True
    assert uri.fromString("tel:+1-816-555-1212") is True
    assert uri.fromString("telnet://192.0.2.16:80/") is True
    assert uri.fromString("urn:oasis:names:specification:docbook:dtd:xml:4.1.2") is True


if __name__ == "__main__":
    pytest.main()



import pytest
import dartpy as dart


def test_simple_frame():
    shape_frame = dart.dynamics.SimpleFrame()
    assert not shape_frame.hasVisualAspect()
    assert shape_frame.getVisualAspect() is None
    assert shape_frame.getVisualAspect(False) is None
    visual = shape_frame.createVisualAspect()
    assert visual is not None


if __name__ == "__main__":
    pytest.main()



import dartpy as dart


class HelloWorldNode(dart.gui.osg.RealTimeWorldNode):
    # Use this function to execute custom code before each time that the
    # window is rendered. This function can be deleted if it does not need
    # to be used.
    def customPreRefresh(self):
        pass

    # Use this function to execute custom code after each time that the
    # window is rendered. This function can be deleted if it does not need
    # to be used.
    def customPostRefresh(self):
        pass

    # Use this function to execute custom code before each simulation time
    # step is performed. This function can be deleted if it does not need
    # to be used.
    def customPreStep(self):
        pass

    # Use this function to execute custom code after each simulation time
    # step is performed. This function can be deleted if it does not need
    # to be used.
    def customPostStep(self):
        pass


def main():
    world = dart.simulation.World()

    urdfParser = dart.utils.DartLoader()
    kr5 = urdfParser.parseSkeleton("dart://sample/urdf/KR5/KR5 sixx R650.urdf")
    ground = urdfParser.parseSkeleton("dart://sample/urdf/KR5/ground.urdf")
    world.addSkeleton(kr5)
    world.addSkeleton(ground)
    world.setGravity([0, -9.81, 0])

    # Create world node and add it to viewer
    node = HelloWorldNode(world)

    # create a viewer with background color (red, green, blue, alpha), here: white
    viewer = dart.gui.osg.Viewer([1.0, 1.0, 1.0, 1.0])
    viewer.addWorldNode(node)

    # Grid settings
    grid = dart.gui.osg.GridVisual()
    grid.setPlaneType(dart.gui.osg.GridVisual.PlaneType.ZX)
    grid.setOffset([0, -0.55, 0])
    viewer.addAttachment(grid)

    viewer.setUpViewInWindow(0, 0, 640, 480)
    viewer.setCameraHomePosition([2.0, 1.0, 2.0],
                                 [0.00, 0.00, 0.00],
                                 [-0.24, 0.94, -0.25])
    viewer.run()


if __name__ == "__main__":
    main()



import platform
import pytest
import numpy as np
import dartpy as dart


def test_basic():
    world_frame = dart.dynamics.Frame.World()
    assert world_frame.isWorld()

    frame1 = dart.dynamics.SimpleFrame()
    assert not frame1.isWorld()
    assert not frame1.isShapeNode()
    assert frame1.isShapeFrame()

    assert frame1.getTransform().translation()[0] == pytest.approx(0)
    assert frame1.getTransform().translation()[1] == pytest.approx(0)
    assert frame1.getTransform().translation()[2] == pytest.approx(0)

    frame1.setTranslation([1, 2, 3])
    assert frame1.getTransform().translation()[0] == pytest.approx(1)
    assert frame1.getTransform().translation()[1] == pytest.approx(2)
    assert frame1.getTransform().translation()[2] == pytest.approx(3)

    assert frame1.getParentFrame().isWorld()
    assert frame1.descendsFrom(None)
    assert frame1.descendsFrom(world_frame)
    assert frame1.descendsFrom(frame1)

    frame2 = frame1.spawnChildSimpleFrame()
    assert not frame2.isWorld()
    assert frame2.descendsFrom(None)
    assert frame2.descendsFrom(world_frame)
    assert frame2.descendsFrom(frame1)
    assert frame2.descendsFrom(frame2)

    assert frame2.getTransform().translation()[0] == pytest.approx(1)
    assert frame2.getTransform().translation()[1] == pytest.approx(2)
    assert frame2.getTransform().translation()[2] == pytest.approx(3)

    frame2.setRelativeTranslation([1, 2, 3])
    assert frame2.getTransform().translation()[0] == pytest.approx(2)
    assert frame2.getTransform().translation()[1] == pytest.approx(4)
    assert frame2.getTransform().translation()[2] == pytest.approx(6)

    frame2.setTranslation([1, 2, 3])
    assert frame2.getTransform().translation()[0] == pytest.approx(1)
    assert frame2.getTransform().translation()[1] == pytest.approx(2)
    assert frame2.getTransform().translation()[2] == pytest.approx(3)


if __name__ == "__main__":
    pytest.main()



import numpy as np
import dartpy as dart


class MyWorldNode(dart.gui.osg.RealTimeWorldNode):
    def __init__(self, world, shadow):
        super(MyWorldNode, self).__init__(world, shadow)

    def customPreStep(self):
        pass


def main():
    world = dart.utils.SkelParser.readWorld("dart://sample/skel/cubes.skel")
    world.setGravity([0, -9.81, 0])


    viewer = dart.gui.osg.Viewer()
    shadow = dart.gui.osg.WorldNode.createDefaultShadowTechnique(viewer)

    node = MyWorldNode(world, shadow)
    viewer.addWorldNode(node)

    viewer.setUpViewInWindow(0, 0, 640, 480)
    viewer.setCameraHomePosition([0.8, 0.0, 0.8], [0, -0.25, 0], [0, 0.5, 0])
    viewer.run()


if __name__== "__main__":
    main()



import contextlib
import os
import re
import sys
import platform
import subprocess
import glob

from codecs import open  # To use a consistent encoding
from setuptools import setup, Extension
from setuptools.command.build_ext import build_ext
from distutils.version import LooseVersion
import distutils.log

# Get the current directory path.
dart_root = os.path.abspath(os.path.dirname(__file__))
dartpy_root = os.path.join(dart_root, 'python')

with open(os.path.join(dart_root, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()
description = 'Python API of Dynamic Animation and Robotics Toolkit.'

distutils.log.set_verbosity(distutils.log.DEBUG)  # Set DEBUG level


class CMakeExtension(Extension):
    def __init__(self, name, sourcedir='', sources=[]):
        Extension.__init__(self, name, sources=sources)
        self.sourcedir = os.path.abspath(sourcedir)


class CMakeBuild(build_ext):
    """ Wrapper class that builds the extension using CMake. """

    def run(self):
        """ Build using CMake from the specified build directory. """
        try:
            out = subprocess.check_output(['cmake', '--version'])
        except OSError:
            raise RuntimeError(
                "CMake must be installed to build the following extensions: " +
                ", ".join(e.name for e in self.extensions))

        if platform.system() == "Windows":
            cmake_version = LooseVersion(
                re.search(r'version\s*([\d.]+)', out.decode()).group(1))
            if cmake_version < '3.10.2':
                raise RuntimeError("CMake >= 3.10.2 is required on Windows")

        distutils.log.set_verbosity(distutils.log.DEBUG)  # Set DEBUG level

        for ext in self.extensions:
            self.build_extension(ext)

    def build_extension(self, ext):
        cmake_args = [
            '-DPYTHON_EXECUTABLE=' + sys.executable
        ]

        cfg = 'Debug' if self.debug else 'Release'
        build_args = ['--config', cfg]

        if platform.system() == "Windows":
            if sys.maxsize > 2**32:
                cmake_args += ['-A', 'x64']
            build_args += ['--', '/m']
        else:
            cmake_args += ['-DCMAKE_BUILD_TYPE=' + cfg]
            build_args += ['--', '-j4']

        env = os.environ.copy()
        env['CXXFLAGS'] = '{} -DVERSION_INFO=\\"{}\\"'.format(
            env.get('CXXFLAGS', ''), self.distribution.get_version())
        if not os.path.exists(self.build_temp):
            os.makedirs(self.build_temp)
        subprocess.check_call(['cmake', ext.sourcedir] + cmake_args,
                              cwd=self.build_temp,
                              env=env)
        subprocess.check_call(['cmake', '--build', '.', '--target', 'dartpy'] +
                              build_args,
                              cwd=self.build_temp)
        subprocess.check_call(['cmake', '--build', '.', '--target', 'install'],
                              cwd=self.build_temp)


sources = ['CMakeLists.txt']
sources.extend(glob.glob('cmake/**/*', recursive=True))
sources.extend(glob.glob('dart/**/*', recursive=True))
sources.extend(glob.glob('python/**/*', recursive=True))
sources.extend(glob.glob('doxygen/**/*', recursive=True))

# Set up the python package wrapping this extension.
setup(
    name='dartpy',
    version='0.0.1-10',
    description=description,
    long_description=long_description,
    long_description_content_type='text/markdown',
    ext_modules=[CMakeExtension('dartpy', sources=sources)],
    url='https://github.com/dartsim/dart',
    author='Jeongseok Lee',
    author_email='jslee02@gmail.com',
    license='BSD 2-Clause',
    keywords='dartsim robotics',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Framework :: Robot Framework',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: BSD License',
        'Topic :: Scientific/Engineering',
    ],
    cmdclass=dict(build_ext=CMakeBuild),
)



import inspect
import os
import os.path


def get_path_to_this_file():
    return inspect.getfile(inspect.currentframe())


def get_asset_path(rel_path, check_existing=False):
    if rel_path.startswith("/"):
        full_path = rel_path
    else:
        full_path = os.path.join(os.path.dirname(__file__), "../../data/", rel_path)
    if check_existing and not os.path.exists(full_path):
        raise IOError("File %s does not exist" % full_path)

    return full_path



import math
import numpy as np
import dartpy as dart


class MyWorldNode(dart.gui.osg.RealTimeWorldNode):
    def __init__(self, world, chain):
        super(MyWorldNode, self).__init__(world)
        self.chain = chain

    def customPreStep(self):
        pass


def main():
    world = dart.utils.SkelParser.readWorld('dart://sample/skel/chain.skel');
    world.setGravity([0, -9.81, 0])
    world.setTimeStep(1.0/2000)

    chain = world.getSkeleton(0)
    for i in range(chain.getNumJoints()):
        joint = chain.getJoint(i)
        for j in range(joint.getNumDofs()):
            joint.setDampingCoefficient(j, 0.01)

    dof = chain.getNumDofs()

    init_pose = np.zeros(dof)
    init_pose[20] = math.pi * 0.4
    init_pose[23] = math.pi * 0.4
    init_pose[26] = math.pi * 0.4
    init_pose[29] = math.pi * 0.4
    chain.setPositions(init_pose)

    # Create a ball joint contraint
    bd1 = chain.getBodyNode('link 6')
    bd2 = chain.getBodyNode('link 10')
    shape_node1 = bd1.getShapeNode(0)
    shape_node2 = bd2.getShapeNode(0)
    visual1 = shape_node1.getVisualAspect()
    visual2 = shape_node2.getVisualAspect()
    visual1.setColor([1, 0, 0])
    visual2.setColor([1, 0, 0])

    offset = [0, 0.025, 0]
    joint_pos = bd1.getTransform().multiply(offset)
    constraint = dart.constraint.BallJointConstraint(bd1, bd2, joint_pos)
    constraint_solver = world.getConstraintSolver()
    constraint_solver.addConstraint(constraint)

    node = MyWorldNode(world, chain)

    # Create world node and add it to viewer
    viewer = dart.gui.osg.Viewer()
    viewer.addWorldNode(node)

    # Grid settings
    grid = dart.gui.osg.GridVisual()
    grid.setPlaneType(dart.gui.osg.GridVisual.PlaneType.ZX)
    grid.setOffset([0, -0.55, 0])
    viewer.addAttachment(grid)

    viewer.setUpViewInWindow(0, 0, 640, 480)
    viewer.setCameraHomePosition([0.6, 0.3, 0.6], [0, -0.2, 0], [0, 1, 0])
    viewer.run()


if __name__ == "__main__":
    main()
